@@ -48,6 +48,7 @@
 #include "rom73.h"
 #include "rom83p.h"
 #include "romdump.h"
+#include "keys83p.h"
 
 // Screen coordinates of the TI83+
 #define TI73_ROWS  64
@@ -72,6 +73,38 @@ static int		send_key	(CalcHandle* handle, uint16_t key)
 	return 0;
 }
 
+static int		execute		(CalcHandle* handle, VarEntry *ve, const char* args)
+{
+	unsigned int i;
+
+	if (handle->model == CALC_TI73 && ve->type == TI73_ASM)
+		return ERR_VOID_FUNCTION;
+
+	// Go back to homescreen
+	PAUSE(200);
+	TRYF(send_key(handle, KEY83P_Quit));
+	TRYF(send_key(handle, KEY83P_Clear));
+	TRYF(send_key(handle, KEY83P_Clear));
+
+	// Launch program by remote control
+	if(ve->type == TI83p_ASM)
+	{
+		TRYF(send_key(handle, KEY83P_Asm));
+	}
+	TRYF(send_key(handle, KEY83P_Exec));
+
+	for(i = 0; i < strlen(ve->name); i++)
+	{
+		const CalcKey *ck = ticalcs_keys_83p((ve->name)[i]);
+		TRYF(send_key(handle, ck->normal.value));
+	}
+
+	TRYF(send_key(handle, KEY83P_Enter));
+	PAUSE(200);
+
+	return 0;
+}
+
 static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitmap)
 {
 	uint16_t max_cnt;
@@ -90,7 +123,7 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	if (err != ERR_CHECKSUM) { TRYF(err) };
 	TRYF(ti73_send_ACK());
 
-	*bitmap = (uint8_t *) malloc(TI73_COLS * TI73_ROWS / 8);
+	*bitmap = (uint8_t *)g_malloc(TI73_COLS * TI73_ROWS / 8);
 	if(*bitmap == NULL)
 		return ERR_MALLOC;
 	memcpy(*bitmap, buf, TI73_COLS * TI73_ROWS  / 8);
@@ -98,22 +131,22 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	return 0;
 }
 
-static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
+static int		get_dirlist	(CalcHandle* handle, GNode** vars, GNode** apps)
 {
 	TreeInfo *ti;
 	uint16_t unused;
 	uint32_t memory;
-	TNode *folder, *root;	
+	GNode *folder, *root;	
 	char *utf8;
 
-	(*apps) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+	(*apps) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = APP_NODE_NAME;
 	(*apps)->data = ti;
 
-	(*vars) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+	(*vars) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = VAR_NODE_NAME;
 	(*vars)->data = ti;
@@ -126,16 +159,40 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 	TRYF(ti73_send_ACK());
 	ti->mem_free = memory;
 
-	folder = t_node_new(NULL);
-	t_node_append(*vars, folder);
+	folder = g_node_new(NULL);
+	g_node_append(*vars, folder);
 
-	root = t_node_new(NULL);
-	t_node_append(*apps, root);
+	root = g_node_new(NULL);
+	g_node_append(*apps, root);
+
+	// Add permanent variables (Window, RclWindow, TblSet aka WINDW, ZSTO, TABLE)
+	{
+		GNode *node;
+		VarEntry *ve;
+
+		ve = tifiles_ve_create();
+		ve->type = TI84p_WINDW;
+		node = g_node_new(ve);
+		g_node_append(folder, node);
+
+		if(handle->model != CALC_TI73)
+		{
+			ve = tifiles_ve_create();
+			ve->type = TI84p_ZSTO;
+			node = g_node_new(ve);
+			g_node_append(folder, node);
+		}
+
+		ve = tifiles_ve_create();
+		ve->type = TI84p_TABLE;
+		node = g_node_new(ve);
+		g_node_append(folder, node);
+	}
 
 	for (;;) 
 	{
 		VarEntry *ve = tifiles_ve_create();
-		TNode *node;
+		GNode *node;
 		int err;
 		uint16_t ve_size;
 
@@ -147,13 +204,13 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 		else if (err != 0)
 			return err;
 
-		node = t_node_new(ve);
+		node = g_node_new(ve);
 		if (ve->type != TI73_APPL)
-			t_node_append(folder, node);
+			g_node_append(folder, node);
 		else
-			t_node_append(root, node);
+			g_node_append(root, node);
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ve->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 		g_snprintf(update_->text, sizeof(update_->text), _("Parsing %s"), utf8);
 		g_free(utf8);
 		update_label();
@@ -319,7 +376,7 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 		  break;
 		}
 
-		utf8 = ticonv_varname_to_utf8(handle->model,entry->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, entry->name, entry->type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -352,7 +409,7 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
     ve = content->entries[0] = tifiles_ve_create();
     memcpy(ve, vr, sizeof(VarEntry));
 
-    utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+    utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
     g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
     update_label();
@@ -386,12 +443,25 @@ static int		recv_var_ns	(CalcHandle* handle, CalcMode mode, FileContent* content
 	return 0;
 }
 
+static int		get_version	(CalcHandle* handle, CalcInfos* infos);
+
 static int		send_flash	(CalcHandle* handle, FlashContent* content)
 {
 	FlashContent *ptr;
 	int i, j, k;
 	int size;
 	char *utf8;
+	//	int se = 1;
+
+	// check for SilverEdition (not useable in boot mode, sic!)
+#if 0
+	{
+		CalcInfos infos = { 0 };
+
+		TRYF(get_version(handle, &infos));
+		se = infos.hw_version & 1;
+	}
+#endif
 
 	// search for data header
 	for (ptr = content; ptr != NULL; ptr = ptr->next)
@@ -425,7 +495,7 @@ static int		send_flash	(CalcHandle* handle, FlashContent* content)
 	ticalcs_info(_("FLASH name: \"%s\""), ptr->name);
 	ticalcs_info(_("FLASH size: %i bytes."), ptr->data_length);
 
-	utf8 = ticonv_varname_to_utf8(handle->model, ptr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, ptr->name, ptr->data_type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -459,13 +529,19 @@ static int		send_flash	(CalcHandle* handle, FlashContent* content)
 			update_->pbar();
 		}
 
-		if(handle->model != CALC_TI84P)
+		/* Note: 
+			TI83+/SE and TI84+/SE don't need a pause (otherwise transfer fails).
+			TI73,TI83+,TI84+ need a pause (otherwise transfer fails).
+		*/
+#if 0
+		if(!se)
 		{
 			if (i == 1)
 			  PAUSE(1000);		// This pause is NEEDED !
 			if (i == ptr->num_pages - 2)
 			  PAUSE(2500);		// This pause is NEEDED !
 		}
+#endif
 	}
 
 	TRYF(ti73_send_EOT());
@@ -489,7 +565,7 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	uint8_t buf[FLASH_PAGE_SIZE + 4];
 	char *utf8;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -694,7 +770,7 @@ static int		set_clock	(CalcHandle* handle, CalcClock* clock)
 	ref.tm_sec = 0;
 	//ref.tm_isdst = 1;
 	r = mktime(&ref);
-	//printf("<%s>\n", asctime(&ref));
+	//printf("%s\n", asctime(&ref));
 
 	cur.tm_year = clock->year - 1900;
 	cur.tm_mon = clock->month - 1;
@@ -704,7 +780,7 @@ static int		set_clock	(CalcHandle* handle, CalcClock* clock)
 	cur.tm_sec = clock->seconds;
 	cur.tm_isdst = 1;
 	c = mktime(&cur);
-	//printf("<%s>\n", asctime(&cur));
+	//printf("%s\n", asctime(&cur));
 	
 	calc_time = (uint32_t)difftime(c, r);
 
@@ -775,11 +851,11 @@ static int		get_clock	(CalcHandle* handle, CalcClock* clock)
 	ref.tm_sec = 0;
 	//ref.tm_isdst = 1;
 	r = mktime(&ref);
-	//printf("<%s>\n", asctime(&ref));
+	//printf("%s\n", asctime(&ref));
 
 	c = r + calc_time;
 	cur = localtime(&c);
-	//printf("<%s>\n", asctime(cur));
+	//printf("%s\n", asctime(cur));
 
 	clock->year = cur->tm_year + 1900;
 	clock->month = cur->tm_mon + 1;
@@ -798,7 +874,7 @@ static int		del_var		(CalcHandle* handle, VarRequest* vr)
 {
 	char *utf8;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), _("Deleting %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -857,7 +933,7 @@ static int		get_version	(CalcHandle* handle, CalcInfos* infos)
 	ticalcs_info(_("  OS: %s"), infos->os_version);
 	ticalcs_info(_("  BIOS: %s"), infos->boot_version);
 	ticalcs_info(_("  HW: %i"), infos->hw_version);
-	ticalcs_info(_("  Battery: %s"), (infos->battery & 1) ? "low" : "good");
+	ticalcs_info(_("  Battery: %s"), infos->battery ? _("good") : _("low"));
 
 	return 0;
 }
@@ -870,7 +946,7 @@ static int		send_cert	(CalcHandle* handle, FlashContent* content)
 
 	// search for cert header
 	for (ptr = content; ptr != NULL; ptr = ptr->next)
-		if(ptr->data_type == TI83p_CERTIF)
+		if(ptr->data_type == TI83p_CERT)
 			break;
 
 	// send content
@@ -917,7 +993,7 @@ static int		recv_cert	(CalcHandle* handle, FlashContent* content)
 
 	content->model = handle->model;
 	strcpy(content->name, "");
-	content->data_type = TI83p_CERTIF;
+	content->data_type = TI83p_CERT;
 	content->device_type = 0x73;
 	content->num_pages = 0;
 	content->data_part = (uint8_t *)tifiles_ve_alloc_data(2 * 1024 * 1024);	// 2MB max
@@ -958,8 +1034,8 @@ const CalcFncts calc_73 =
 {
 	CALC_TI73,
 	"TI73",
-	N_("TI-73"),
-	N_("TI-73"),
+	"TI-73",
+	"TI-73",
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | 
 	OPS_FLASH | OPS_IDLIST | OPS_ROMDUMP | OPS_VERSION | OPS_OS |
 	FTS_SILENT | FTS_MEMFREE | FTS_FLASH | FTS_BACKUP,
@@ -967,6 +1043,7 @@ const CalcFncts calc_73 =
 		"2P", "1L", "2P1L", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
@@ -995,15 +1072,16 @@ const CalcFncts calc_83p =
 {
 	CALC_TI83P,
 	"TI83+",
-	N_("TI-83 Plus"),
-	N_("TI-83 Plus"),
+	"TI-83 Plus",
+	"TI-83 Plus",
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | 
-	OPS_FLASH | OPS_IDLIST | OPS_ROMDUMP | OPS_CLOCK | OPS_DELVAR | OPS_VERSION | OPS_OS |
+	OPS_FLASH | OPS_IDLIST | OPS_ROMDUMP | OPS_DELVAR | OPS_VERSION | OPS_OS |
 	FTS_SILENT | FTS_MEMFREE | FTS_FLASH | FTS_CERT | FTS_BACKUP,
 	{"", "", "1P", "1L", "", "2P", "2P", "2P1L", "1P1L", "2P1L", "1P1L", "2P1L", "2P1L",
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
@@ -1032,8 +1110,8 @@ const CalcFncts calc_84p =
 {
 	CALC_TI84P,
 	"TI84+",
-	N_("TI-84 Plus"),
-	N_("TI-84 Plus"),
+	"TI-84 Plus",
+	"TI-84 Plus",
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | 
 	OPS_FLASH | OPS_IDLIST | OPS_ROMDUMP | OPS_CLOCK | OPS_DELVAR | OPS_VERSION | OPS_OS |
 	FTS_SILENT | FTS_MEMFREE | FTS_FLASH | FTS_CERT | FTS_BACKUP,
@@ -1041,6 +1119,7 @@ const CalcFncts calc_84p =
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
