@@ -1,5 +1,5 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: calc_84p.c 3074 2006-11-12 13:12:06Z roms $ */
+/* $Id: calc_84p.c 3536 2007-06-26 14:57:07Z roms $ */
 
 /*  libticalcs2 - hand-helds support library, a part of the TiLP project
  *  Copyright (c) 1999-2005  Romain Lievin
@@ -44,6 +44,8 @@
 #include "dusb_vpkt.h"
 #include "dusb_cmd.h"
 
+#include "keys83p.h"
+
 // Screen coordinates of the TI83+
 #define TI84P_ROWS  64
 #define TI84P_COLS  96
@@ -61,6 +63,27 @@ static int		is_ready	(CalcHandle* handle)
 
 static int		send_key	(CalcHandle* handle, uint16_t key)
 {
+	TRYF(cmd_s_execute(handle, "", "", EID_KEY, NULL, key));
+	TRYF(cmd_r_delay_ack(handle));
+	TRYF(cmd_r_data_ack(handle));
+
+	return 0;
+}
+
+static int		execute		(CalcHandle* handle, VarEntry *ve, const char* args)
+{
+	uint8_t action;
+
+	switch(ve->type)
+	{
+	case TI84p_ASM:  action = EID_ASM; break;
+	case TI84p_APPL: action = EID_APP; break;
+	default:		 action = EID_PRGM; break;
+	}
+
+	TRYF(cmd_s_execute(handle, ve->folder, ve->name, action, args, 0));
+	TRYF(cmd_r_data_ack(handle));
+
 	return 0;
 }
 
@@ -80,7 +103,7 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	if(!param[0]->ok)
 		return ERR_INVALID_PACKET;
 	
-	*bitmap = (uint8_t *) malloc(TI84P_COLS * TI84P_ROWS / 8);
+	*bitmap = (uint8_t *)g_malloc(TI84P_COLS * TI84P_ROWS / 8);
 	if(*bitmap == NULL) 
 		return ERR_MALLOC;
 	memcpy(*bitmap, param[0]->data, TI84P_COLS * TI84P_ROWS / 8);
@@ -89,40 +112,64 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	return 0;
 }
 
-static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
+static int		get_dirlist	(CalcHandle* handle, GNode** vars, GNode** apps)
 {
 	uint16_t aids[] = { AID_VAR_SIZE, AID_VAR_TYPE, AID_ARCHIVED, };
 	const int size = sizeof(aids) / sizeof(uint16_t);
 	TreeInfo *ti;
 	int err;
 	CalcAttr **attr;
-	TNode *folder, *root;	
+	GNode *folder, *root;	
 	char fldname[40], varname[40];
 	char *utf8;
 
-	(*apps) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+	(*apps) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = APP_NODE_NAME;
 	(*apps)->data = ti;
 
-	(*vars) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+	(*vars) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = VAR_NODE_NAME;
 	(*vars)->data = ti;
 
-	folder = t_node_new(NULL);
-	t_node_append(*vars, folder);
+	folder = g_node_new(NULL);
+	g_node_append(*vars, folder);
 
-	root = t_node_new(NULL);
-	t_node_append(*apps, root);
+	root = g_node_new(NULL);
+	g_node_append(*apps, root);
+
+	// Add permanent variables (Window, RclWindow, TblSet aka WINDW, ZSTO, TABLE)
+	{
+		GNode *node;
+		VarEntry *ve;
+
+		ve = tifiles_ve_create();
+		strcpy(ve->name, "Window");
+		ve->type = TI84p_WINDW;
+		node = g_node_new(ve);
+		g_node_append(folder, node);
+
+		ve = tifiles_ve_create();
+		strcpy(ve->name, "RclWin");
+		ve->type = TI84p_ZSTO;
+		node = g_node_new(ve);
+		g_node_append(folder, node);
+
+		ve = tifiles_ve_create();
+		strcpy(ve->name, "TblSet");
+		ve->type = TI84p_TABLE;
+		node = g_node_new(ve);
+		g_node_append(folder, node);
+	}
 
 	TRYF(cmd_s_dirlist_request(handle, size, aids));
 	do
 	{
 		VarEntry *ve = tifiles_ve_create();
-		TNode *node;
+		GNode *node;
 
 		attr = ca_new_array(size);
 		err = cmd_r_var_header(handle, fldname, varname, attr);
@@ -137,13 +184,13 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 		ve->attr = attr[2]->data[0] ? ATTRB_ARCHIVED : ATTRB_NONE;
 		ca_del_array(size, attr);
 
-		node = t_node_new(ve);
+		node = g_node_new(ve);
 		if (ve->type != TI73_APPL)
-			t_node_append(folder, node);
+			g_node_append(folder, node);
 		else
-			t_node_append(root, node);
+			g_node_append(root, node);
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ve->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 		g_snprintf(update_->text, sizeof(update_->text), _("Parsing %s"), utf8);
 		g_free(utf8);
 		update_label();
@@ -183,7 +230,7 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 		if(ve->action == ACT_SKIP)
 			continue;
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ve->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -210,7 +257,7 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 
 static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, VarRequest* vr)
 {
-	uint16_t aids[] = { AID_ARCHIVED, AID_VAR_VERSION };
+	uint16_t aids[] = { AID_ARCHIVED, AID_VAR_VERSION, AID_VAR_SIZE };
 	const int naids = sizeof(aids) / sizeof(uint16_t);
 	CalcAttr **attrs;
 	const int nattrs = 1;
@@ -219,7 +266,7 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
 	VarEntry *ve;
 	char *utf8;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -231,7 +278,7 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
 
 	TRYF(cmd_s_var_request(handle,"",vr->name,naids,aids,nattrs,CA(attrs)));
 	ca_del_array(nattrs, attrs);
-	attrs = ca_new_array(nattrs);
+	attrs = ca_new_array(naids);
 	TRYF(cmd_r_var_header(handle, fldname, varname, attrs));
 	TRYF(cmd_r_var_content(handle, NULL, &data));
 
@@ -243,11 +290,12 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
     ve = content->entries[0] = tifiles_ve_create();
     memcpy(ve, vr, sizeof(VarEntry));
 
+	ve->size = GINT32_FROM_BE(*((uint32_t *)(attrs[2]->data)));
 	ve->data = tifiles_ve_alloc_data(ve->size);
 	memcpy(ve->data, data, ve->size);
 	
-	free(data);	
-	ca_del_array(nattrs, attrs);
+	g_free(data);	
+	ca_del_array(naids, attrs);
 	return 0;
 }
 
@@ -325,7 +373,7 @@ static int		send_flash	(CalcHandle* handle, FlashContent* content)
 	}
 
 	// send
-	utf8 = ticonv_varname_to_utf8(handle->model, ptr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, ptr->name, ptr->data_type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -361,7 +409,7 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	uint16_t data_page = 0;
 	int r, q;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -374,7 +422,7 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	TRYF(cmd_s_var_request(handle, "", vr->name, naids, aids, 
 			       nattrs, CA(attrs)));
 	ca_del_array(nattrs, attrs);
-	attrs = ca_new_array(nattrs);
+	attrs = ca_new_array(naids);
 	TRYF(cmd_r_var_header(handle, fldname, varname, attrs));
 	TRYF(cmd_r_var_content(handle, NULL, &data));
 
@@ -420,8 +468,8 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	}
 	content->num_pages = page+1;
 
-	free(data);	
-	ca_del_array(nattrs, attrs);
+	g_free(data);	
+	ca_del_array(naids, attrs);
 	return 0;
 }
 
@@ -562,7 +610,7 @@ static int		recv_idlist	(CalcHandle* handle, uint8_t* id)
 	TRYF(cmd_s_var_request(handle, "", "IDList", naids, aids, 
 			       nattrs, CA(attrs)));
 	ca_del_array(nattrs, attrs);
-	attrs = ca_new_array(nattrs);
+	attrs = ca_new_array(naids);
 	TRYF(cmd_r_var_header(handle, folder, name, attrs));
 	TRYF(cmd_r_var_content(handle, (uint32_t *)&varsize, &data));
 
@@ -574,8 +622,8 @@ static int		recv_idlist	(CalcHandle* handle, uint8_t* id)
 		sprintf((char *)&id[2 * (i-4)], "%02x", data[i]);
 	id[7*2] = '\0';
 
-	free(data);
-	ca_del_array(nattrs, attrs);
+	g_free(data);
+	ca_del_array(naids, attrs);
 	return 0;
 }
 
@@ -716,7 +764,7 @@ static int		del_var		(CalcHandle* handle, VarRequest* vr)
 	const int size = 2;
 	char *utf8;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), _("Deleting %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -855,16 +903,17 @@ extern int tixx_recv_backup(CalcHandle* handle, BackupContent* content);
 const CalcFncts calc_84p_usb = 
 {
 	CALC_TI84P_USB,
-	"TI84+ (USB)",
-	N_("TI-84 Plus thru DirectLink USB"),
-	N_("TI-84 Plus thru DirectLink USB"),
+	"TI84+",
+	"TI-84 Plus",
+	N_("TI-84 Plus thru DirectLink"),
 	OPS_ISREADY | OPS_SCREEN | OPS_DIRLIST | OPS_VARS | OPS_FLASH | OPS_OS |
-	OPS_IDLIST | OPS_CLOCK | OPS_DELVAR | OPS_VERSION | OPS_BACKUP | 
+	OPS_IDLIST | OPS_CLOCK | OPS_DELVAR | OPS_VERSION | OPS_BACKUP | OPS_KEYS |
 	FTS_SILENT | FTS_MEMFREE | FTS_FLASH,
 	{"", "", "1P", "1L", "", "2P", "2P", "2P1L", "1P1L", "2P1L", "1P1L", "2P1L", "2P1L",
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
