@@ -66,6 +66,44 @@ static int		send_key	(CalcHandle* handle, uint16_t key)
 {
 	TRYF(ti89_send_KEY(key));
 	TRYF(ti89_recv_ACK(&key));
+	PAUSE(50);
+
+	return 0;
+}
+
+static int		execute		(CalcHandle* handle, VarEntry *ve, const char* args)
+{
+	unsigned int i;
+
+	if(ve->type == TI89_APPL)
+		return ERR_VOID_FUNCTION;
+
+	// Go back to homescreen
+	PAUSE(200);
+	TRYF(send_key(handle, KEY89_HOME));
+	TRYF(send_key(handle, KEY89_CLEAR));
+	TRYF(send_key(handle, KEY89_CLEAR));
+
+	// Launch program by remote control
+	for(i = 0; i < strlen(ve->folder); i++)
+		TRYF(send_key(handle, (ve->folder)[i]));
+
+    if(strcmp(ve->folder, ""))
+		TRYF(send_key(handle, '\\'));
+
+	for(i = 0; i < strlen(ve->name); i++)
+		TRYF(send_key(handle, (ve->name)[i]));
+
+	TRYF(send_key(handle, KEY89_LP));
+	if(args)
+	{
+		for(i = 0; i < strlen(args); i++)
+			TRYF(send_key(handle, args[i]));
+	}
+    TRYF(send_key(handle, KEY89_RP));
+
+    TRYF(send_key(handle, KEY89_ENTER));
+	PAUSE(200);
 
 	return 0;
 }
@@ -103,7 +141,7 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	if(err != ERR_CHECKSUM) { TRYF(err) };
 	TRYF(ti89_send_ACK());
 
-	*bitmap = (uint8_t *) malloc(TI89_COLS * TI89_ROWS / 8);
+	*bitmap = (uint8_t *)g_malloc(TI89_COLS * TI89_ROWS / 8);
 	if(*bitmap == NULL) 
 		return ERR_MALLOC;
 	memcpy(*bitmap, buf, TI89_COLS * TI89_ROWS / 8);
@@ -122,7 +160,7 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	return 0;
 }
 
-static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
+static int		get_dirlist	(CalcHandle* handle, GNode** vars, GNode** apps)
 {
 	TreeInfo *ti;
 	VarEntry info;
@@ -130,22 +168,22 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 	uint8_t buffer[65536];
     int i, j;
     uint8_t extra = (handle->model == CALC_V200) ? 8 : 0;
-	TNode *root;
+	GNode *root;
 
-	(*apps) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+	(*apps) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = APP_NODE_NAME;
 	(*apps)->data = ti;
 
-    (*vars) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+    (*vars) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = VAR_NODE_NAME;
 	(*vars)->data = ti;
 
-	root = t_node_new(NULL);
-	t_node_append(*apps, root);
+	root = g_node_new(NULL);
+	g_node_append(*apps, root);
 
     TRYF(ti89_send_REQ(TI89_FDIR << 24, TI89_RDIR, ""));
     TRYF(ti89_recv_ACK(NULL));
@@ -165,7 +203,7 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 	for(j = 4; j < (int)block_size;) 
 	{
 		VarEntry *fe = tifiles_ve_create();
-        TNode *node;
+        GNode *node;
 
         memcpy(fe->name, buffer + j, 8);
         fe->name[8] = '\0';
@@ -183,8 +221,8 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 
         if(fe->type == TI89_DIR)
 		{
-			node = t_node_new(fe);
-			t_node_append(*vars, node);
+			node = g_node_new(fe);
+			g_node_append(*vars, node);
 		}
         else if(fe->type == TI89_APPL)
 		{
@@ -194,9 +232,9 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 	}
 
 	// get list of variables into each folder
-	for(i = 0; i < (int)t_node_n_children(*vars); i++) 
+	for(i = 0; i < (int)g_node_n_children(*vars); i++) 
 	{
-		TNode *folder = t_node_nth_child(*vars, i);
+		GNode *folder = g_node_nth_child(*vars, i);
 		char *folder_name = ((VarEntry *) (folder->data))->name;
 		char *u1, *u2;
 
@@ -220,7 +258,7 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 		for(j = 4 + 14 + extra; j < (int)block_size;) 
 		{
 			VarEntry *ve = tifiles_ve_create();
-			TNode *node;
+			GNode *node;
 
 			memcpy(ve->name, buffer + j, 8);
 			ve->name[8] = '\0';
@@ -236,19 +274,23 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 			ve->attr,
 			ve->size);
 
-			u1 = ticonv_varname_to_utf8(handle->model, ((VarEntry *) (folder->data))->name);
-			u2 = ticonv_varname_to_utf8(handle->model, ve->name);
+			u1 = ticonv_varname_to_utf8(handle->model, ((VarEntry *) (folder->data))->name, -1);
+			u2 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 			g_snprintf(update_->text, sizeof(update_->text), _("Parsing %s/%s"), u1, u2);
 			g_free(u1); g_free(u2);
 			update_label();
 
 			if(ve->type == TI89_APPL) 
 			{
-				if(!ticalcs_dirlist_ve_exist(*apps, ve->name))
+				VarEntry arg;
+
+				memset(&arg, 0, sizeof(arg));
+				strcpy(arg.name, ve->name);
+				if(!ticalcs_dirlist_ve_exist(*apps, &arg))
 				{
 					strcpy(ve->folder, "");
-					node = t_node_new(ve);
-					t_node_append(root, node);
+					node = g_node_new(ve);
+					g_node_append(root, node);
 				}
 			} 
 			else
@@ -260,8 +302,8 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 				}
 				else
 				{
-					node = t_node_new(ve);
-					t_node_append(folder, node);
+					node = g_node_new(ve);
+					g_node_append(folder, node);
 				}
 			}
 		}
@@ -308,7 +350,7 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 			tifiles_build_fullname(handle->model, varname, entry->folder, entry->name);
 		}
 
-		utf8 = ticonv_varname_to_utf8(handle->model, varname);
+		utf8 = ticonv_varname_to_utf8(handle->model, varname, vartype);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -354,7 +396,7 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
 	memcpy(ve, vr, sizeof(VarEntry));
 
 	tifiles_build_fullname(handle->model, varname, vr->folder, vr->name);
-	utf8 = ticonv_varname_to_utf8(handle->model, varname);
+	utf8 = ticonv_varname_to_utf8(handle->model, varname, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -433,7 +475,7 @@ static int		send_var_ns	(CalcHandle* handle, CalcMode mode, FileContent* content
 			tifiles_build_fullname(handle->model, varname, entry->folder, entry->name);
 		}
 
-		utf8 = ticonv_varname_to_utf8(handle->model, varname);
+		utf8 = ticonv_varname_to_utf8(handle->model, varname, vartype);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -502,7 +544,7 @@ static int		recv_var_ns	(CalcHandle* handle, CalcMode mode, FileContent* content
             strcpy(ve->name, tipath);
         }
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ve->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -541,7 +583,7 @@ static int		send_flash	(CalcHandle* handle, FlashContent* content)
 		ticalcs_info(_("FLASH name: \"%s\""), ptr->name);
 		ticalcs_info(_("FLASH size: %i bytes."), ptr->data_length);
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ptr->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ptr->name, ptr->data_type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -602,7 +644,7 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	int i;
 	char *utf8;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -687,6 +729,12 @@ static int		recv_idlist	(CalcHandle* handle, uint8_t* idlist)
 
 static int		dump_rom_1	(CalcHandle* handle)
 {
+	// Go back to homescreen
+	PAUSE(200);
+	TRYF(send_key(handle, KEY89_HOME));
+	TRYF(send_key(handle, KEY89_CLEAR));
+	TRYF(send_key(handle, KEY89_CLEAR));
+
 	// Send dumping program
 	TRYF(rd_send(handle, "romdump.89z", romDumpSize89, romDump89));
 	PAUSE(1000);
@@ -698,13 +746,6 @@ static int		dump_rom_1	(CalcHandle* handle)
 static int		dump_rom_2	(CalcHandle* handle, CalcDumpSize size, const char *filename)
 {
 	// Launch program by remote control
-	PAUSE(200);
-	TRYF(send_key(handle, KEY89_HOME));
-	PAUSE(50);
-	TRYF(send_key(handle, KEY89_CLEAR));
-	PAUSE(50);
-	TRYF(send_key(handle, KEY89_CLEAR));
-	PAUSE(50);
     TRYF(send_key(handle, 'm'));
     TRYF(send_key(handle, 'a'));
     TRYF(send_key(handle, 'i'));
@@ -805,7 +846,7 @@ static int		del_var		(CalcHandle* handle, VarRequest* vr)
 	char *utf8;
 
 	tifiles_build_fullname(handle->model, varname, vr->folder, vr->name);
-	utf8 = ticonv_varname_to_utf8(handle->model, varname);
+	utf8 = ticonv_varname_to_utf8(handle->model, varname, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), _("Deleting %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -824,7 +865,7 @@ static int		new_folder  (CalcHandle* handle, VarRequest* vr)
 	char *utf8;
 
 	tifiles_build_fullname(handle->model, varname, vr->folder, "a1234567");
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->folder);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->folder, -1);
 	g_snprintf(update_->text, sizeof(update_->text), _("Creating %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -933,8 +974,8 @@ const CalcFncts calc_89 =
 {
 	CALC_TI89,
 	"TI89",
-	N_("TI-89"),
-	N_("TI-89"),
+	"TI-89",
+	"TI-89",
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | 
 	OPS_FLASH | OPS_IDLIST | OPS_CLOCK | OPS_ROMDUMP |
 	OPS_DELVAR | OPS_NEWFLD | OPS_VERSION |
@@ -943,6 +984,7 @@ const CalcFncts calc_89 =
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
@@ -971,8 +1013,8 @@ const CalcFncts calc_92p =
 {
 	CALC_TI92P,
 	"TI92+",
-	N_("TI-92 Plus"),
-	N_("TI-92 Plus"),
+	"TI-92 Plus",
+	"TI-92 Plus",
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | 
 	OPS_FLASH | OPS_IDLIST | OPS_CLOCK | OPS_ROMDUMP |
 	OPS_DELVAR | OPS_NEWFLD | OPS_VERSION | OPS_OS |
@@ -981,6 +1023,7 @@ const CalcFncts calc_92p =
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
@@ -1008,9 +1051,9 @@ const CalcFncts calc_92p =
 const CalcFncts calc_89t = 
 {
 	CALC_TI89T,
-	"TI89t",
-	N_("TI-89 Titanium"),
-	N_("TI-89 Titanium"),
+	"Titanium",
+	"TI-89 Titanium",
+	"TI-89 Titanium",
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | 
 	OPS_FLASH | OPS_IDLIST | OPS_CLOCK | OPS_ROMDUMP | 
 	OPS_DELVAR | OPS_NEWFLD | OPS_VERSION | OPS_OS |
@@ -1019,6 +1062,7 @@ const CalcFncts calc_89t =
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
@@ -1047,7 +1091,7 @@ const CalcFncts calc_v2 =
 {
 	CALC_V200,
 	"V200",
-	N_("V200PLT"),
+	"V200PLT",
 	N_("V200 Portable Learning Tool"),
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | 
 	OPS_FLASH | OPS_IDLIST | OPS_CLOCK | OPS_ROMDUMP |
@@ -1057,6 +1101,7 @@ const CalcFncts calc_v2 =
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
