@@ -1,5 +1,5 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: calc_89t.c 3054 2006-11-06 17:10:19Z roms $ */
+/* $Id: calc_89t.c 3536 2007-06-26 14:57:07Z roms $ */
 
 /*  libticalcs2 - hand-helds support library, a part of the TiLP project
  *  Copyright (C) 1999-2005  Romain Lievin
@@ -46,6 +46,7 @@
 
 #include "rom89t.h"
 #include "romdump.h"
+#include "keys89.h"
 
 // Screen coordinates of the Titanium
 #define TI89T_ROWS			128
@@ -65,6 +66,27 @@ static int		is_ready	(CalcHandle* handle)
 
 static int		send_key	(CalcHandle* handle, uint16_t key)
 {
+	PAUSE(25);	// this pause is needed between 2 keys
+	TRYF(cmd_s_execute(handle, "", "", EID_KEY, NULL, key));
+	TRYF(cmd_r_data_ack(handle));
+
+	return 0;
+}
+
+static int		execute		(CalcHandle* handle, VarEntry *ve, const char *args)
+{
+	uint8_t action;
+
+	switch(ve->type)
+	{
+	case TI89t_ASM:  action = EID_ASM; break;
+	case TI89t_APPL: action = EID_APP; break;
+	default:		 action = EID_PRGM; break;
+	}
+
+	TRYF(cmd_s_execute(handle, ve->folder, ve->name, action, args, 0));
+	TRYF(cmd_r_data_ack(handle));
+
 	return 0;
 }
 
@@ -84,7 +106,7 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	if(!param[0]->ok)
 		return ERR_INVALID_PACKET;
 	
-	*bitmap = (uint8_t *) malloc(TI89T_COLS * TI89T_ROWS / 8);
+	*bitmap = (uint8_t *)g_malloc(TI89T_COLS * TI89T_ROWS / 8);
 	if(*bitmap == NULL) 
 		return ERR_MALLOC;
 	memcpy(*bitmap, param[0]->data, TI89T_COLS * TI89T_ROWS / 8);
@@ -101,39 +123,39 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	return 0;
 }
 
-static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
+static int		get_dirlist	(CalcHandle* handle, GNode** vars, GNode** apps)
 {
 	uint16_t aids[] = { AID_VAR_TYPE, AID_ARCHIVED, AID_4APPVAR, AID_VAR_SIZE, AID_LOCKED, AID_UNKNOWN_42 };
 	const int size = sizeof(aids) / sizeof(uint16_t);
 	TreeInfo *ti;
 	int err;
 	CalcAttr **attr;
-	TNode *root, *folder = NULL;
+	GNode *root, *folder = NULL;
 	char fldname[40];
 	char varname[40];
 	char folder_name[40] = "";
 	char *u1, *u2;
 
-	(*apps) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+	(*apps) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = APP_NODE_NAME;
 	(*apps)->data = ti;
 
-    (*vars) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+    (*vars) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = VAR_NODE_NAME;
 	(*vars)->data = ti;
 
-	root = t_node_new(NULL);
-	t_node_append(*apps, root);
+	root = g_node_new(NULL);
+	g_node_append(*apps, root);
 
 	TRYF(cmd_s_dirlist_request(handle, size, aids));
 	for(;;)
 	{
 		VarEntry *ve = tifiles_ve_create();
-		TNode *node;
+		GNode *node;
 
 		attr = ca_new_array(size);
 		err = cmd_r_var_header(handle, fldname, varname, attr);
@@ -155,8 +177,8 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 			strcpy(ve->name, ve->folder);
 			strcpy(ve->folder, "");
 			
-			node = t_node_new(ve);
-			folder = t_node_append(*vars, node);
+			node = g_node_new(ve);
+			folder = g_node_append(*vars, node);
 		}
 		else
 		{
@@ -164,11 +186,11 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 				tifiles_ve_delete(ve);
 			else
 			{
-				node = t_node_new(ve);
+				node = g_node_new(ve);
 				if (ve->type != TI73_APPL)
-					t_node_append(folder, node);
+					g_node_append(folder, node);
 				else
-					t_node_append(root, node);
+					g_node_append(root, node);
 			}
 		}
 /*
@@ -178,8 +200,8 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 			ve->attr,
 			ve->size);
 */
-		u1 = ticonv_varname_to_utf8(handle->model, ((VarEntry *) (folder->data))->name);
-		u2 = ticonv_varname_to_utf8(handle->model, ve->name);
+		u1 = ticonv_varname_to_utf8(handle->model, ((VarEntry *) (folder->data))->name, -1);
+		u2 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 			g_snprintf(update_->text, sizeof(update_->text), _("Parsing %s/%s"), u1, u2);
 			g_free(u1); g_free(u2);
 			update_label();
@@ -211,6 +233,7 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 	char *utf8;
 	CalcAttr **attrs;
 	const int nattrs = 4;
+	uint32_t pkt_size;
 
 	update_->cnt2 = 0;
 	update_->max2 = content->num_entries;
@@ -222,7 +245,7 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 		if(ve->action == ACT_SKIP)
 			continue;
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ve->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -238,9 +261,17 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 		attrs[3] = ca_new(AID_LOCKED, 1);
 		attrs[3]->data[0] = ve->attr == ATTRB_LOCKED ? 1 : 0;
 
-		TRYF(cmd_s_rts(handle, ve->folder, ve->name, ve->size, 
-			       nattrs, CA(attrs)));
+		TRYF(cmd_s_rts(handle, ve->folder, ve->name, ve->size, nattrs, CA(attrs)));
 		TRYF(cmd_r_data_ack(handle));
+
+		if((ve->size < dusb_get_buf_size()) && (ve->size > 255))
+		{
+			pkt_size = 0xff;
+
+			TRYF(dusb_send_buf_size_request(handle, pkt_size));
+			TRYF(dusb_recv_buf_size_alloc(handle, &pkt_size));
+		}
+
 		TRYF(cmd_s_var_content(handle, ve->size, ve->data));
 		TRYF(cmd_r_data_ack(handle));
 		TRYF(cmd_s_eot(handle));
@@ -266,7 +297,7 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
 	VarEntry *ve;
 	char *utf8;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -279,7 +310,7 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
 	TRYF(cmd_s_var_request(handle, vr->folder, vr->name, naids, aids, 
 			       nattrs, CA(attrs)));
 	ca_del_array(nattrs, attrs);
-	attrs = ca_new_array(nattrs);
+	attrs = ca_new_array(naids);
 	TRYF(cmd_r_var_header(handle, fldname, varname, attrs));
 	TRYF(cmd_r_var_content(handle, NULL, &data));
 
@@ -293,9 +324,9 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
 
 	ve->data = tifiles_ve_alloc_data(ve->size);
 	memcpy(ve->data, data, ve->size);
-	free(data);	
+	g_free(data);	
 
-	ca_del_array(nattrs, attrs);
+	ca_del_array(naids, attrs);
 	return 0;
 }
 
@@ -331,7 +362,7 @@ static int		send_flash	(CalcHandle* handle, FlashContent* content)
 		ticalcs_info(_("FLASH name: \"%s\""), ptr->name);
 		ticalcs_info(_("FLASH size: %i bytes."), ptr->data_length);
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ptr->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ptr->name, ptr->data_type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -368,7 +399,7 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	uint8_t *data;
 	char *utf8;
 	
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -381,7 +412,7 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	TRYF(cmd_s_var_request(handle, "", vr->name, naids, aids, 
 			       nattrs, CA(attrs)));
 	ca_del_array(nattrs, attrs);
-	attrs = ca_new_array(nattrs);
+	attrs = ca_new_array(naids);
 	TRYF(cmd_r_var_header(handle, fldname, varname, attrs));
 	TRYF(cmd_r_var_content(handle, NULL, &data));
 
@@ -393,9 +424,9 @@ static int		recv_flash	(CalcHandle* handle, FlashContent* content, VarRequest* v
 	content->device_type = DEVICE_TYPE_89;
 
 	memcpy(content->data_part, data, content->data_length);
-	free(data);
+	g_free(data);
 
-	ca_del_array(nattrs, attrs);
+	ca_del_array(naids, attrs);
 	return 0;
 }
 
@@ -489,6 +520,15 @@ static int		recv_idlist	(CalcHandle* handle, uint8_t* id)
 
 static int		dump_rom_1	(CalcHandle* handle)
 {
+	CalcParam *param;
+
+	// Go back to HOME screen
+	param = cp_new(PID_HOMESCREEN, 1);
+	param->data[0] = 1;
+	TRYF(cmd_s_param_set(handle, param));
+	TRYF(cmd_r_data_ack(handle));
+	cp_del(param);
+
 	// Send dumping program
 	TRYF(rd_send(handle, "romdump.89z", romDumpSize89t, romDump89t));
 	PAUSE(1000);
@@ -498,23 +538,9 @@ static int		dump_rom_1	(CalcHandle* handle)
 
 static int		dump_rom_2	(CalcHandle* handle, CalcDumpSize size, const char *filename)
 {
-	int err;
-
-	// Wait for user's action (execing program)
-	sprintf(handle->updat->text, _("Waiting for user's action..."));
-	handle->updat->label();
-
-	do
-	{
-		handle->updat->refresh();
-		if (handle->updat->cancel)
-			return ERR_ABORT;
-		
-		//send RDY request ???
-		PAUSE(1500);
-		err = rd_is_ready(handle);
-	}
-	while (err == ERROR_READ_TIMEOUT);
+	// Launch program by remote control	
+    TRYF(cmd_s_execute(handle, "main", "romdump", EID_ASM, "", 0));
+	TRYF(cmd_r_data_ack(handle));
 
 	// Get dump
 	TRYF(rd_dump(handle, filename));
@@ -652,7 +678,7 @@ static int		del_var		(CalcHandle* handle, VarRequest* vr)
 	char *utf8;
 
 	tifiles_build_fullname(handle->model, varname, vr->folder, vr->name);
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->name);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->name, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), _("Deleting %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -681,7 +707,7 @@ static int		new_folder  (CalcHandle* handle, VarRequest* vr)
 	const int nattrs = 4;
 	char *utf8;
 
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->folder);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->folder, -1);
 	g_snprintf(update_->text, sizeof(update_->text), _("Creating %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -838,16 +864,17 @@ extern int tixx_recv_backup(CalcHandle* handle, BackupContent* content);
 const CalcFncts calc_89t_usb = 
 {
 	CALC_TI89T_USB,
-	"Titanium (USB)",
-	N_("TI-89 Titanium thru DirectLink USB"),
-	N_("TI-89 Titanium thru DirectLink USB"),
+	"Titanium",
+	"TI-89 Titanium",
+	N_("TI-89 Titanium thru DirectLink"),
 	OPS_ISREADY | OPS_SCREEN | OPS_DIRLIST | OPS_VARS | OPS_FLASH | OPS_OS | OPS_ROMDUMP |
-	OPS_IDLIST | OPS_CLOCK | OPS_DELVAR | OPS_NEWFLD | OPS_VERSION | OPS_BACKUP | 
+	OPS_IDLIST | OPS_CLOCK | OPS_DELVAR | OPS_NEWFLD | OPS_VERSION | OPS_BACKUP | OPS_KEYS |
 	FTS_SILENT | FTS_MEMFREE | FTS_FLASH | FTS_FOLDER,
 	{"", "", "1P", "1L", "", "2P1L", "2P1L", "2P1L", "1P1L", "2P1L", "1P1L", "2P1L", "2P1L",
 		"2P", "1L", "2P", "", "", "1L", "1L", "", "1L", "1L" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
