@@ -64,10 +64,45 @@ static int		send_key	(CalcHandle* handle, uint16_t key)
 {
 	TRYF(ti92_send_KEY(key));
 	TRYF(ti92_recv_ACK(&key));
+	PAUSE(50);
 	
 	return 0;
 }
 
+static int		execute		(CalcHandle* handle, VarEntry *ve, const char* args)
+{
+	unsigned int i;
+
+	// Go back to homescreen
+	PAUSE(200);
+	TRYF(send_key(handle, (KEY92P_CTRL + KEY92P_q)));
+	TRYF(send_key(handle, KEY92P_CLEAR));
+	TRYF(send_key(handle, KEY92P_CLEAR));
+
+	// Launch program by remote control
+	for(i = 0; i < strlen(ve->folder); i++)
+		TRYF(send_key(handle, (ve->folder)[i]));
+
+    if(strcmp(ve->folder, ""))
+		TRYF(send_key(handle, '\\'));
+
+	for(i = 0; i < strlen(ve->name); i++)
+		TRYF(send_key(handle, (ve->name)[i]));
+
+    TRYF(send_key(handle, KEY92P_LP));
+	if(args)
+	{
+		for(i = 0; i < strlen(args); i++)
+			TRYF(send_key(handle, args[i]));
+	}
+    TRYF(send_key(handle, KEY92P_RP));
+
+    TRYF(send_key(handle, KEY92P_ENTER));
+	PAUSE(200);
+
+	return 0;
+}
+
 static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitmap)
 {
 	uint32_t max_cnt;
@@ -86,7 +121,7 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	if (err != ERR_CHECKSUM) { TRYF(err) };
 	TRYF(ti92_send_ACK());
 
-	*bitmap = (uint8_t *) malloc(TI92_COLS * TI92_ROWS / 8);
+	*bitmap = (uint8_t *)g_malloc(TI92_COLS * TI92_ROWS / 8);
 	if(*bitmap == NULL)
 		return ERR_MALLOC;
 	memcpy(*bitmap, buf, TI92_COLS * TI92_ROWS / 8);
@@ -94,7 +129,7 @@ static int		recv_screen	(CalcHandle* handle, CalcScreenCoord* sc, uint8_t** bitm
 	return 0;
 }
 
-static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
+static int		get_dirlist	(CalcHandle* handle, GNode** vars, GNode** apps)
 {
 	TreeInfo *ti;
 	VarEntry info;
@@ -102,18 +137,18 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 	uint8_t buffer[65536];
 	int err;
 	char folder_name[9] = "";
-	TNode *folder = NULL;
+	GNode *folder = NULL;
 	char *utf8;
 
 	// get list of folders & FLASH apps
-    (*vars) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+    (*vars) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = VAR_NODE_NAME;
 	(*vars)->data = ti;
 
-	(*apps) = t_node_new(NULL);
-	ti = (TreeInfo *)malloc(sizeof(TreeInfo));
+	(*apps) = g_node_new(NULL);
+	ti = (TreeInfo *)g_malloc(sizeof(TreeInfo));
 	ti->model = handle->model;
 	ti->type = APP_NODE_NAME;
 	(*apps)->data = ti;
@@ -125,7 +160,7 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 	for (;;) 
 	{
 		VarEntry *ve = tifiles_ve_create();
-		TNode *node;
+		GNode *node;
 
 		TRYF(ti92_send_ACK());
 		TRYF(ti92_send_CTS());
@@ -143,8 +178,8 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 		if (ve->type == TI92_DIR) 
 		{
 			strcpy(folder_name, ve->name);
-			node = t_node_new(ve);
-			folder = t_node_append(*vars, node);
+			node = g_node_new(ve);
+			folder = g_node_append(*vars, node);
 		} 
 		else 
 		{
@@ -157,8 +192,8 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 			}
 			else
 			{
-				node = t_node_new(ve);
-				t_node_append(folder, node);
+				node = g_node_new(ve);
+				g_node_append(folder, node);
 			}
 		}
 
@@ -174,7 +209,7 @@ static int		get_dirlist	(CalcHandle* handle, TNode** vars, TNode** apps)
 			break;
 		TRYF(err);
 
-		utf8 = ticonv_varname_to_utf8(handle->model,ve->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 		g_snprintf(update_->text, sizeof(update_->text), _("Parsing %s/%s"),
 			((VarEntry *) (folder->data))->name, utf8);
 		g_free(utf8);
@@ -291,7 +326,7 @@ static int		send_var	(CalcHandle* handle, CalcMode mode, FileContent* content)
 		else 
 			tifiles_build_fullname(handle->model, varname, entry->folder, entry->name);
 
-		utf8 = ticonv_varname_to_utf8(handle->model, varname);
+		utf8 = ticonv_varname_to_utf8(handle->model, varname, entry->type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -336,7 +371,7 @@ static int		recv_var	(CalcHandle* handle, CalcMode mode, FileContent* content, V
 
 	tifiles_build_fullname(handle->model, varname, vr->folder, vr->name);
 
-	utf8 = ticonv_varname_to_utf8(handle->model, varname);
+	utf8 = ticonv_varname_to_utf8(handle->model, varname, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 	g_free(utf8);
 	update_label();
@@ -408,7 +443,7 @@ static int		recv_var_ns	(CalcHandle* handle, CalcMode mode, FileContent* content
             strcpy(ve->name, tipath);
         }
 
-		utf8 = ticonv_varname_to_utf8(handle->model, ve->name);
+		utf8 = ticonv_varname_to_utf8(handle->model, ve->name, ve->type);
 		g_snprintf(update_->text, sizeof(update_->text), "%s", utf8);
 		g_free(utf8);
 		update_label();
@@ -449,6 +484,12 @@ static int		recv_idlist	(CalcHandle* handle, uint8_t* idlist)
 
 static int		dump_rom_1	(CalcHandle* handle)
 {
+	// Go back to homescreen
+	PAUSE(200);
+	TRYF(send_key(handle, (KEY92P_CTRL + KEY92P_q)));
+	TRYF(send_key(handle, KEY92P_CLEAR));
+	TRYF(send_key(handle, KEY92P_CLEAR));
+
 	// Send dumping program
 	TRYF(rd_send(handle, "romdump.92p", romDumpSize92, romDump92));
 	PAUSE(1000);
@@ -460,11 +501,6 @@ static int		dump_rom_1	(CalcHandle* handle)
 static int		dump_rom_2	(CalcHandle* handle, CalcDumpSize size, const char *filename)
 {
 	// Launch program by remote control
-	PAUSE(200);
-	TRYF(send_key(handle, KEY92P_CLEAR));
-	PAUSE(50);
-	TRYF(send_key(handle, KEY92P_CLEAR));
-	PAUSE(50);
     TRYF(send_key(handle, 'm'));
     TRYF(send_key(handle, 'a'));
     TRYF(send_key(handle, 'i'));
@@ -505,7 +541,7 @@ static int		del_var		(CalcHandle* handle, VarRequest* vr)
 	char *utf8;
 
 	tifiles_build_fullname(handle->model, varname, vr->folder, vr->name);
-	utf8 = ticonv_varname_to_utf8(handle->model, varname);
+	utf8 = ticonv_varname_to_utf8(handle->model, varname, vr->type);
 	g_snprintf(update_->text, sizeof(update_->text), _("Deleting %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -542,7 +578,7 @@ static int		new_folder  (CalcHandle* handle, VarRequest* vr)
 	char *utf8;
 
 	tifiles_build_fullname(handle->model, varname, vr->folder, "a1234567");
-	utf8 = ticonv_varname_to_utf8(handle->model, vr->folder);
+	utf8 = ticonv_varname_to_utf8(handle->model, vr->folder, -1);
 	g_snprintf(update_->text, sizeof(update_->text), _("Creating %s..."), utf8);
 	g_free(utf8);
 	update_label();
@@ -604,8 +640,8 @@ const CalcFncts calc_92 =
 {
 	CALC_TI92,
 	"TI92",
-	N_("TI-92"),
-	N_("TI-92"),
+	"TI-92",
+	"TI-92",
 	OPS_ISREADY | OPS_KEYS | OPS_SCREEN | OPS_DIRLIST | OPS_BACKUP | OPS_VARS | OPS_ROMDUMP |
 	OPS_DELVAR | OPS_NEWFLD | OPS_VERSION |
 	FTS_SILENT | FTS_FOLDER | FTS_BACKUP,
@@ -613,6 +649,7 @@ const CalcFncts calc_92 =
 	"", "", "2P", "", "", "1L", "1L", "", "", "" },
 	&is_ready,
 	&send_key,
+	&execute,
 	&recv_screen,
 	&get_dirlist,
 	&get_memfree,
