@@ -43,11 +43,11 @@ GList *cpca_list = NULL;
 
 CalcParam*	cp_new(uint16_t id, uint16_t size)
 {
-	CalcParam* cp = calloc(1, sizeof(CalcParam));
+	CalcParam* cp = g_malloc0(sizeof(CalcParam));
 
 	cp->id = id;
 	cp->size = size;
-	cp->data = calloc(1, size);
+	cp->data = g_malloc0(size);
 
 	cpca_list = g_list_append(cpca_list, cp);
 	return cp;
@@ -57,13 +57,13 @@ void		cp_del(CalcParam* cp)
 {
 	cpca_list = g_list_remove(cpca_list, cp);
 
-	free(cp->data);
-	free(cp);
+	g_free(cp->data);
+	g_free(cp);
 }
 
 CalcParam** cp_new_array(int size)
 {
-	CalcParam** array = calloc(size+1, sizeof(CalcParam *));
+	CalcParam** array = g_malloc0((size+1) * sizeof(CalcParam *));
 	return array;
 }
 
@@ -73,18 +73,18 @@ void cp_del_array(int size, CalcParam **params)
 
 	for(i = 0; i < size && params[i]; i++)
 		cp_del(params[i]);
-	free(params);
+	g_free(params);
 }
 
 /////////////----------------
 
 CalcAttr*	ca_new(uint16_t id, uint16_t size)
 {
-	CalcAttr* cp = calloc(1, sizeof(CalcAttr));
+	CalcAttr* cp = g_malloc0(sizeof(CalcAttr));
 
 	cp->id = id;
 	cp->size = size;
-	cp->data = calloc(1, size);
+	cp->data = g_malloc0(size);
 
 	cpca_list = g_list_append(cpca_list, cp);
 	return cp;
@@ -94,13 +94,13 @@ void		ca_del(CalcAttr* cp)
 {
 	cpca_list = g_list_remove(cpca_list, cp);
 
-	free(cp->data);
-	free(cp);
+	g_free(cp->data);
+	g_free(cp);
 }
 
 CalcAttr** ca_new_array(int size)
 {
-	CalcAttr** array = calloc(size+1, sizeof(CalcParam *));
+	CalcAttr** array = g_malloc0((size+1) * sizeof(CalcParam *));
 	return array;
 }
 
@@ -110,7 +110,7 @@ void ca_del_array(int size, CalcAttr **attrs)
 
 	for(i = 0; i < size && attrs[i]; i++)
 		ca_del(attrs[i]);
-	free(attrs);
+	g_free(attrs);
 }
 
 void cpca_purge(void)
@@ -151,8 +151,9 @@ static void byteswap(uint8_t *data, uint32_t len)
 /////////////----------------
 
 static uint16_t usb_errors[] = { 
-	0x0004, 0x0006, 0x0008, 0x0009, 0x000c, 0x000e, 
-	0x0012, 0x001c, 0x001d, 0x0022, 0x002b };
+	0x0004, 0x0006, 0x0008, 0x0009, 0x000c, 0x000d, 0x000e, 
+	0x0011, 0x0012, 0x001c, 0x001d, 0x0022, 0x0029, 0x002b, 
+	0x0034 };
 
 static int err_code(VirtualPacket *pkt)
 {
@@ -207,7 +208,6 @@ int cmd_s_mode_set(CalcHandle *h, ModeSet mode)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Ping / Set Mode");
 	ticalcs_info("   %04x %04x %04x %04x %04x", mode.arg1, mode.arg2, mode.arg3, mode.arg4, mode.arg5);
 
 	return 0;
@@ -226,7 +226,6 @@ int cmd_s_os_begin(CalcHandle *h, uint32_t size)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: OS Transfer Begin");
 	ticalcs_info("   size = %08x (%i)", size, size);
 
 	return 0;
@@ -250,7 +249,6 @@ int cmd_r_os_ack(CalcHandle *h, uint32_t *size)
 	*size = (pkt->data[0] << 24) | (pkt->data[1] << 16) | (pkt->data[2] << 8) | (pkt->data[3] << 0);
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: OS Transfer Ack");
 	ticalcs_info("   size = %08x (%i)", *size, *size);
 
 	return 0;
@@ -270,7 +268,6 @@ static int s_os(uint8_t type, CalcHandle *h, uint16_t addr, uint8_t page, uint8_
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: %s", type == VPKT_OS_HEADER ? "OS Header" : "OS Data");
 	ticalcs_info("   addr=%04x, page=%02x, flag=%02x, size=%04x", addr, page, flag, size);
 
 	return 0;
@@ -298,7 +295,6 @@ int cmd_s_os_header_89(CalcHandle *h, uint32_t size, uint8_t *data)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: OS Header");
 	ticalcs_info("   size = %08x (%i)", size, size);
 
 	return 0;
@@ -314,7 +310,6 @@ int cmd_s_os_data_89(CalcHandle *h, uint32_t size, uint8_t *data)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: OS Data");
 	ticalcs_info("   size = %08x (%i)", size, size);
 
 	return 0;
@@ -336,7 +331,6 @@ int cmd_r_eot_ack(CalcHandle *h)
 		return ERR_INVALID_PACKET;
 	
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: EOT Ack");
 
 	return 0;
 }
@@ -361,7 +355,6 @@ int cmd_s_param_request(CalcHandle *h, int npids, uint16_t *pids)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Parameter Request");
 	ticalcs_info("   npids=%i", npids);
 
 	return 0;
@@ -395,14 +388,13 @@ int cmd_r_param_data(CalcHandle *h, int nparams, CalcParam **params)
 		if(s->ok)
 		{
 			s->size = pkt->data[j++] << 8; s->size |= pkt->data[j++];
-			s->data = (uint8_t *)calloc(1, s->size);
+			s->data = (uint8_t *)g_malloc0(s->size);
 			memcpy(s->data, &pkt->data[j], s->size);
 			j += s->size;
 		}
 	}
 	
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: Parameter Data");
 	ticalcs_info("   nparams=%i", nparams);
 
 	return 0;
@@ -436,14 +428,13 @@ int cmd_s_dirlist_request(CalcHandle *h, int naids, uint16_t *aids)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Directory Listing");
 	ticalcs_info("   naids=%i", naids);
 
 	return 0;
 }
 
 // 0x000A: variable header (name is utf-8)
-// beware: attr array is allocated by function
+// beware: attr array contents is allocated by function
 int cmd_r_var_header(CalcHandle *h, char *folder, char *name, CalcAttr **attr)
 {
 	VirtualPacket* pkt;
@@ -495,15 +486,14 @@ int cmd_r_var_header(CalcHandle *h, char *folder, char *name, CalcAttr **attr)
 		if(s->ok)
 		{
 			s->size = pkt->data[j++] << 8; s->size |= pkt->data[j++];
-			s->data = (uint8_t *)calloc(1, s->size);
+			s->data = (uint8_t *)g_malloc0(s->size);
 			memcpy(s->data, &pkt->data[j], s->size);
 			j += s->size;
 		}
 	}
 	
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: Variable Header");
-	ticalcs_info("   folder=<%s>, name=<%s>", folder, name);
+	ticalcs_info("   folder=%s, name=%s", folder, name);
 
 	return 0;
 }
@@ -557,8 +547,7 @@ int cmd_s_rts(CalcHandle *h, const char *folder, const char *name, uint32_t size
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Request to Send");
-	ticalcs_info("   folder=<%s>, name=<%s>, size=%i, nattrs=%i", folder, name, size, nattrs);
+	ticalcs_info("   folder=%s, name=%s, size=%i, nattrs=%i", folder, name, size, nattrs);
 
 	return 0;
 }
@@ -621,8 +610,7 @@ int cmd_s_var_request(CalcHandle *h, const char *folder, const char *name,
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Request Variable");
-	ticalcs_info("   folder=<%s>, name=<%s>, naids=%i, nattrs=%i", folder, name, naids, nattrs);
+	ticalcs_info("   folder=%s, name=%s, naids=%i, nattrs=%i", folder, name, naids, nattrs);
 
 	return 0;
 }
@@ -645,11 +633,10 @@ int cmd_r_var_content(CalcHandle *h, uint32_t *size, uint8_t **data)
 	if(size != NULL)
 		*size = pkt->size;
 
-	*data = calloc(pkt->size, 1);
+	*data = g_malloc0(pkt->size);
 	memcpy(*data, pkt->data, pkt->size);
 	
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Variable Contents");
 	ticalcs_info("   size=%i", size);
 
 	return 0;
@@ -666,7 +653,6 @@ int cmd_s_var_content(CalcHandle *h, uint32_t size, uint8_t *data)
 	TRYF(dusb_send_data(h, pkt));
 	
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Variable Contents");
 	ticalcs_info("   size=%i", size);
 
 	return 0;
@@ -741,8 +727,64 @@ int cmd_s_var_delete(CalcHandle *h, const char *folder, const char *name, int na
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Delete Variable");
-	ticalcs_info("   folder=<%s>, name=<%s>, nattrs=%i", folder, name, nattrs);
+	ticalcs_info("   folder=%s, name=%s, nattrs=%i", folder, name, nattrs);
+
+	return 0;
+}
+
+// 0x0011: remote control
+int cmd_s_execute(CalcHandle *h, const char *folder, const char *name, 
+					uint8_t action, const char *args, uint16_t code)
+{
+	VirtualPacket* pkt;
+	int pks;
+	int j = 0;
+
+	pks = 3;
+	if(args) pks += strlen(args); else pks += 2;
+	if(strlen(folder)) pks += strlen(folder)+1;
+	if(strlen(name)) pks += strlen(name)+1;
+	pkt = vtl_pkt_new(pks, VPKT_EXECUTE);
+
+	pkt->data[j++] = strlen(folder);
+	if(strlen(folder))
+	{		
+		memcpy(pkt->data + j, folder, strlen(folder)+1);
+		j += strlen(folder)+1;
+	}
+
+	pkt->data[j++] = strlen(name);
+	if(strlen(name))
+	{		
+		memcpy(pkt->data + j, name, strlen(name)+1);
+		j += strlen(name)+1;
+	}
+	
+	pkt->data[j++] = action;
+
+	if(action != EID_KEY && args != NULL)
+		memcpy(pkt->data + j, args, strlen(args));
+	else if(action == EID_KEY || args == NULL)
+	{
+		if(h->model == CALC_TI89T_USB)
+		{
+			pkt->data[j++] = MSB(code);
+			pkt->data[j++] = LSB(code);
+		}
+		else if(h->model == CALC_TI84P_USB)
+		{
+			pkt->data[j++] = LSB(code);
+			pkt->data[j++] = MSB(code);
+		}
+	}
+
+	TRYF(dusb_send_data(h, pkt));
+
+	vtl_pkt_del(pkt);
+	if(args)
+		ticalcs_info("   action=%i, folder=%s, name=%s, args=%s", action, folder, name, args);
+	else
+		ticalcs_info("   action=%i, keycode=%04x", action, code);
 
 	return 0;
 }
@@ -763,7 +805,6 @@ int cmd_r_mode_ack(CalcHandle *h)
 		return ERR_INVALID_PACKET;
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: Mode Setting Ack");
 
 	return 0;
 }
@@ -784,7 +825,6 @@ int cmd_r_data_ack(CalcHandle *h)
 		return ERR_INVALID_PACKET;
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: Data Ack");
 
 	return 0;
 }
@@ -805,7 +845,6 @@ int cmd_r_delay_ack(CalcHandle *h)
 	PAUSE(100);
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: Delay Ack");
 
 	return 0;
 }
@@ -819,7 +858,6 @@ int cmd_s_eot(CalcHandle *h)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: End of Transmission");
 
 	return 0;
 }
@@ -840,7 +878,6 @@ int cmd_r_eot(CalcHandle *h)
 		return ERR_INVALID_PACKET;
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  TI->PC: End of Transmission");
 
 	return 0;
 }
@@ -857,7 +894,6 @@ int cmd_s_error(CalcHandle *h, uint16_t code)
 	TRYF(dusb_send_data(h, pkt));
 
 	vtl_pkt_del(pkt);
-	//ticalcs_info("  PC->TI: Error");
 	ticalcs_info("   code = %04x", code);
 
 	return 0;
