@@ -1,5 +1,5 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: probe.c 2956 2006-09-24 17:11:22Z roms $ */
+/* $Id: probe.c 3455 2007-06-24 08:18:05Z roms $ */
 
 /*  libticalcs - Ti Calculator library, a part of the TiLP project
  *  Copyright (C) 1999-2005  Romain Livin
@@ -198,7 +198,7 @@ static int ticalcs_probe_calc_2(CalcHandle* handle, CalcModel* model)
  * @type: the calculator model
  *
  * Check if the calculator is ready and detect the type.
- * Works only with FLASH calculators and a AMS2.08 (?!) mini by requesting
+ * Works only on FLASH calculators with an AMS2.08 or OS2.00 by requesting the
  * version. A previous version was based on MID but TI83+/84+, TI89/TI89t, TI92+/V200 
  * could not be distinguished ;-(
  *
@@ -327,7 +327,7 @@ TIEXPORT3 int TICALL ticalcs_probe_calc  (CableHandle* cable, CalcModel* model)
 	memset(&calc, 0, sizeof(CalcHandle));
 	calc.model = *model = CALC_NONE;
 	calc.updat = (CalcUpdate *)&default_update;
-	calc.priv2 = (uint8_t *)malloc(65536 + 4);
+	calc.priv2 = (uint8_t *)g_malloc(65536 + 4);
 	calc.cable = cable;
 	calc.open = !0;
 
@@ -335,7 +335,7 @@ TIEXPORT3 int TICALL ticalcs_probe_calc  (CableHandle* cable, CalcModel* model)
 	err = ticalcs_probe_calc_1(&calc, model);
 	if(!err && (*model != CALC_NONE))
 	{
-		free(calc.priv2);
+		g_free(calc.priv2);
 		return 0;
 	}
 
@@ -343,11 +343,11 @@ TIEXPORT3 int TICALL ticalcs_probe_calc  (CableHandle* cable, CalcModel* model)
 	err = ticalcs_probe_calc_2(&calc, model);
 	if(err)
 	{
-		free(calc.priv2);
+		g_free(calc.priv2);
 		return err;
 	}
 
-	free(calc.priv2);
+	g_free(calc.priv2);
 	return (*model == CALC_NONE) ? ERR_NO_CALC : 0;
 }
 
@@ -372,7 +372,7 @@ TIEXPORT3 int TICALL ticalcs_probe_usb_calc(CableHandle* cable, CalcModel* model
 	memset(&calc, 0, sizeof(CalcHandle));
 	calc.model = *model = CALC_NONE;
 	calc.updat = (CalcUpdate *)&default_update;
-	calc.priv2 = (uint8_t *)malloc(65536 + 4);
+	calc.priv2 = (uint8_t *)g_malloc(65536 + 4);
 	calc.cable = cable;
 	calc.open = !0;
 
@@ -400,7 +400,7 @@ TIEXPORT3 int TICALL ticalcs_probe_usb_calc(CableHandle* cable, CalcModel* model
 		}
 	}
 
-	free(calc.priv2);
+	g_free(calc.priv2);
 	return ret;
 }
 
@@ -432,7 +432,7 @@ TIEXPORT3 int TICALL ticalcs_probe(CableModel c_model, CablePort c_port, CalcMod
 	memset(&calc, 0, sizeof(CalcHandle));
 	calc.model = *model = CALC_NONE;
 	calc.updat = (CalcUpdate *)&default_update;
-	calc.priv2 = (uint8_t *)malloc(65536 + 4);
+	calc.priv2 = (uint8_t *)g_malloc(65536 + 4);
 	calc.cable = handle;
 	calc.open = !0;
 
