@@ -1,5 +1,5 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: ticalcs.h 3074 2006-11-12 13:12:06Z roms $ */
+/* $Id: ticalcs.h 3525 2007-06-26 13:31:39Z roms $ */
 
 /*  libticables - hand-helds support library, a part of the TiLP project
  *  Copyright (C) 1999-2005  Romain Lievin
@@ -30,7 +30,7 @@
 #include <tifiles.h>
 
 #include "export3.h"
-#include "tnode.h"	// to replace by glib !
+#include <glib.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -43,13 +43,13 @@ extern "C" {
 /* Versioning */
 
 #ifdef __WIN32__
-# define LIBCALCS_VERSION "1.0.2"
+# define LIBCALCS_VERSION "1.0.7"
 #else
 # define LIBCALCS_VERSION VERSION
 #endif
 
-#define LIBCALCS_REQUIRES_LIBFILES_VERSION  "0.0.5"	// useless with pkg-config
-#define LIBCALCS_REQUIRES_libticalcs2_VERSION "0.0.1"	// useless with pkg-config
+#define LIBCALCS_REQUIRES_LIBFILES_VERSION		"1.0.7"	// useless with pkg-config
+#define LIBCALCS_REQUIRES_LIBTICABLES_VERSION	"1.0.4"	// useless with pkg-config
 
 /* Types */
 
@@ -487,7 +487,7 @@ typedef struct
 } CalcInfos;
 
 /**
- * TiCable:
+ * CalcFncts:
  * @model: link cable model (CalcModel).
  * @name: name of hand-held like "TI89"
  * @fullname: complete name of hand-held like "TI-89"
@@ -500,6 +500,7 @@ typedef struct
  * @flash: calculator has FLASH ROM
  * @is_ready: check whether calculator is ready
  * @send_key: send key value
+ * @execute: remotely execute a program or application
  * @recv_screen: request a screendump
  * @get_dirlist: request a listing of variables, folders (if any) and apps (if any)
  * @recv_backup: request a backup
@@ -533,10 +534,11 @@ struct _CalcFncts
 	int		(*is_ready)		(CalcHandle*);
 
 	int		(*send_key)		(CalcHandle*, uint16_t);
+	int		(*execute)		(CalcHandle*, VarEntry*, const char*);
 
 	int		(*recv_screen)	(CalcHandle*, CalcScreenCoord*, uint8_t**);
 
-	int		(*get_dirlist)	(CalcHandle*, TNode** vars, TNode** apps);
+	int		(*get_dirlist)	(CalcHandle*, GNode** vars, GNode** apps);
 	int		(*get_memfree)	(CalcHandle*, uint32_t* ram, uint32_t* flash);
 
 	int		(*send_backup)	(CalcHandle*, BackupContent*);
@@ -655,11 +657,14 @@ typedef struct
 	TIEXPORT3 CalcFeatures TICALL ticalcs_calc_features(CalcHandle*);
 
 	TIEXPORT3 int TICALL ticalcs_calc_isready(CalcHandle*);
+
 	TIEXPORT3 int TICALL ticalcs_calc_send_key(CalcHandle*, uint16_t);
+	TIEXPORT3 int TICALL ticalcs_calc_execute(CalcHandle*, VarEntry*, const char*);
+
 	TIEXPORT3 int TICALL ticalcs_calc_recv_screen(CalcHandle *, CalcScreenCoord* sc,
 												 uint8_t** bitmap);
 
-	TIEXPORT3 int TICALL ticalcs_calc_get_dirlist(CalcHandle* handle, TNode** vars, TNode **apps);
+	TIEXPORT3 int TICALL ticalcs_calc_get_dirlist(CalcHandle* handle, GNode** vars, GNode **apps);
 	TIEXPORT3 int TICALL ticalcs_calc_get_memfree(CalcHandle* handle, uint32_t* ram, uint32_t *flash);
 
 	TIEXPORT3 int TICALL ticalcs_calc_send_backup(CalcHandle*, BackupContent*);
@@ -715,16 +720,16 @@ typedef struct
 	TIEXPORT3 int TICALL ticalcs_calc_recv_tigroup2(CalcHandle*, const char*, TigMode);
 
 	// dirlist.c
-	TIEXPORT3 void TICALL ticalcs_dirlist_destroy(TNode** tree);
-	TIEXPORT3 void TICALL ticalcs_dirlist_display(TNode*  tree);
+	TIEXPORT3 void TICALL ticalcs_dirlist_destroy(GNode** tree);
+	TIEXPORT3 void TICALL ticalcs_dirlist_display(GNode*  tree);
 
-	TIEXPORT3 int TICALL ticalcs_dirlist_ram_used(TNode* tree);
-	TIEXPORT3 int TICALL ticalcs_dirlist_flash_used(TNode* vars, TNode* apps);
+	TIEXPORT3 int TICALL ticalcs_dirlist_ram_used(GNode* tree);
+	TIEXPORT3 int TICALL ticalcs_dirlist_flash_used(GNode* vars, GNode* apps);
 
-	TIEXPORT3 int TICALL ticalcs_dirlist_ve_count(TNode* tree);
-	TIEXPORT3 VarEntry *TICALL ticalcs_dirlist_ve_exist(TNode* tree, char* name);
-	TIEXPORT3 void TICALL ticalcs_dirlist_ve_add(TNode* tree, VarEntry *entry);
-	TIEXPORT3 void TICALL ticalcs_dirlist_ve_del(TNode* tree, VarEntry *entry);
+	TIEXPORT3 int TICALL ticalcs_dirlist_ve_count(GNode* tree);
+	TIEXPORT3 VarEntry *TICALL ticalcs_dirlist_ve_exist(GNode* tree, VarEntry *entry);
+	TIEXPORT3 void TICALL ticalcs_dirlist_ve_add(GNode* tree, VarEntry *entry);
+	TIEXPORT3 void TICALL ticalcs_dirlist_ve_del(GNode* tree, VarEntry *entry);
 
 	// type2str.c
 	TIEXPORT3 const char*  TICALL ticalcs_model_to_string(CalcModel model);
@@ -745,10 +750,12 @@ typedef struct
 	TIEXPORT3 int		 TICALL ticalcs_clock_show(CalcModel model, CalcClock* s);
 
 	// tikeys.c
-    TIEXPORT3 CalcKey TICALL ticalcs_keys_73 (uint8_t ascii_code);
-    TIEXPORT3 CalcKey TICALL ticalcs_keys_83p(uint8_t ascii_code);
-	TIEXPORT3 CalcKey TICALL ticalcs_keys_89 (uint8_t ascii_code);
-	TIEXPORT3 CalcKey TICALL ticalcs_keys_92p(uint8_t ascii_code);
+    TIEXPORT3 const CalcKey* TICALL ticalcs_keys_73 (uint8_t ascii_code);
+	TIEXPORT3 const CalcKey* TICALL ticalcs_keys_83 (uint8_t ascii_code);
+    TIEXPORT3 const CalcKey* TICALL ticalcs_keys_83p(uint8_t ascii_code);
+	TIEXPORT3 const CalcKey* TICALL ticalcs_keys_86 (uint8_t ascii_code);
+	TIEXPORT3 const CalcKey* TICALL ticalcs_keys_89 (uint8_t ascii_code);
+	TIEXPORT3 const CalcKey* TICALL ticalcs_keys_92p(uint8_t ascii_code);
 
 	//probe.c
 	TIEXPORT3 int TICALL ticalcs_probe_calc  (CableHandle* cable, CalcModel* model);
